class ElementContainer {
    constructor(shadowHost, shadowRoot) {
        this.shadowHost = shadowHost;
        this.shadowRoot = shadowRoot;
        this.isOpen = false;
    }
    insertSelf() {
        if (!this.isOpen) {
            document.body.appendChild(this.shadowHost);
            this.isOpen = true;
        }
    }
    removeSelf() {
        if (this.isOpen) {
            document.body.removeChild(this.shadowHost);
            this.isOpen = false;
        }
    }
    getContentElement() {
        return this.shadowRoot.querySelector('.modal-body');
    }
    addCloseButtonListener(onClose) {
        // Remember there are 2 close buttons
        this.shadowRoot.querySelectorAll('*[closeButton]')
            .forEach(item => item.addEventListener('click', onClose));
    }
    addConfirmButtonListener(onConfirm) {
        this.shadowRoot.querySelector('.btn.btn-primary[confirmButton]')
            .addEventListener('click', onConfirm);
    }
}

const templateHTML = `

<link 
    href="https://cdn.jsdelivr.net/npm/bootstrap@5.1.3/dist/css/bootstrap.min.css" 
    rel="stylesheet" 
    integrity="sha384-1BmE4kWBq78iYhFldvKuhfTAU6auU8tT94WrHftjDbrCEXSU1oBoqyl2QvZ6jIW3" 
    crossorigin="anonymous" 
/>

<div class="modal fade show" tabindex="-1" style="display: block; background-color: RGBA(0, 0, 0, 0.5);">
    <div class="modal-dialog">
        <div class="modal-content">
        
            <div class="modal-header">
                <h5 class="modal-title">$TITLE</h5>
                <button type="button" class="btn-close" closeButton></button>
            </div>
            
            <div class="modal-body">
                $CONTENT
            </div>
            
            <div class="modal-footer">
                <button type="button" class="btn btn-secondary" closeButton>Close</button>
                <button type="button" class="btn btn-primary" confirmButton>Confirm</button>
            </div>
            
        </div>
    </div>
</div>

`;

class ElementContainerFactory {
    constructor(config) {
        this.config = config;
    }
    createContainer() {
        const root = document.createElement('div');
        ElementContainerFactory.setRootStyles(root);
        const shadowRoot = root.attachShadow({ mode: "closed" });
        this.mountTemplate(shadowRoot);
        const container = new ElementContainer(root, shadowRoot);
        if (this.config.content instanceof HTMLElement) {
            container.getContentElement().appendChild(this.config.content);
        }
        return container;
    }
    static setRootStyles(root) {
        root.style.all = 'initial';
    }
    mountTemplate(shadowRoot) {
        let html = templateHTML.replace('$TITLE', this.config.header);
        if (typeof this.config.content === 'string') {
            html = html.replace('$CONTENT', this.config.content);
        }
        else {
            html = html.replace('$CONTENT', '');
        }
        shadowRoot.innerHTML = html;
    }
}

/**
 * This class listens to the escape key and closes this.host if esc is pressed.
 */
class EscListener {
    constructor(host) {
        this.host = host;
        this.onKeyPress = (e) => {
            if (e.key === 'Escape') {
                this.host.close();
            }
        };
    }
    engage() {
        document.addEventListener('keyup', this.onKeyPress);
    }
    disengage() {
        document.removeEventListener('keyup', this.onKeyPress);
    }
}

class ObjectOrientedBootstrapModal {
    constructor(config) {
        this.escListener = new EscListener(this);
        this.onConfirm = () => {
            // meant to be handled by subclasses
        };
        this.container = new ElementContainerFactory(config).createContainer();
        this.container.addConfirmButtonListener(() => this.onConfirm());
        this.container.addCloseButtonListener(this.close.bind(this));
    }
    open() {
        this.container.insertSelf();
        this.escListener.engage();
    }
    close() {
        this.container.removeSelf();
        this.escListener.disengage();
    }
    get content() {
        return this.container.getContentElement();
    }
}

export { ObjectOrientedBootstrapModal };
